"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const aws = __importStar(require("aws-sdk"));
const uuid_1 = require("uuid");
// 256KiB
const MAX_SQS_MESSAGE_SIZE = 256 * 1024;
class SqsProducer {
    constructor(options) {
        if (options.sqs) {
            this.sqs = options.sqs;
        }
        else {
            this.sqs = new aws.SQS({
                region: options.region,
                endpoint: options.sqsEndpointUrl,
            });
        }
        if (options.largePayloadThoughS3 || options.allPayloadThoughS3) {
            if (!options.s3Bucket) {
                throw new Error('Need to specify "s3Bucket" option when using allPayloadThoughS3 or  largePayloadThoughS3.');
            }
            if (options.s3) {
                this.s3 = options.s3;
            }
            else {
                this.s3 = new aws.S3({
                    region: options.region,
                    endpoint: options.s3EndpointUrl,
                });
            }
        }
        this.queueUrl = options.queueUrl;
        this.largePayloadThoughS3 = options.largePayloadThoughS3;
        this.allPayloadThoughS3 = options.allPayloadThoughS3;
        this.s3Bucket = options.s3Bucket;
    }
    static create(options) {
        return new SqsProducer(options);
    }
    async sendJSON(message, options = {}) {
        const messageBody = JSON.stringify(message);
        const msgSize = Buffer.byteLength(messageBody, 'utf-8');
        if ((msgSize > MAX_SQS_MESSAGE_SIZE && this.largePayloadThoughS3) || this.allPayloadThoughS3) {
            const payloadId = uuid_1.v4();
            const payloadKey = `${payloadId}.json`;
            const s3Response = await this.s3
                .upload({
                Bucket: this.s3Bucket,
                Body: messageBody,
                Key: payloadKey,
                ContentType: 'application/json',
            })
                .promise();
            const sqsResponse = await this.sqs
                .sendMessage({
                QueueUrl: this.queueUrl,
                MessageBody: JSON.stringify({
                    S3Payload: {
                        Id: payloadId,
                        Bucket: s3Response.Bucket,
                        Key: s3Response.Key,
                        Location: s3Response.Location,
                    },
                }),
                DelaySeconds: options.DelaySeconds,
                MessageDeduplicationId: options.MessageDeduplicationId,
                MessageGroupId: options.MessageGroupId,
            })
                .promise();
            return {
                s3Response,
                sqsResponse,
            };
        }
        else if (msgSize > MAX_SQS_MESSAGE_SIZE) {
            throw new Error("Message is too big. Use 'largePayloadThoughS3' option to send large payloads though S3.");
        }
        const sqsResponse = await this.sqs
            .sendMessage({
            QueueUrl: this.queueUrl,
            MessageBody: messageBody,
            DelaySeconds: options.DelaySeconds,
            MessageDeduplicationId: options.MessageDeduplicationId,
            MessageGroupId: options.MessageGroupId,
        })
            .promise();
        return {
            sqsResponse,
        };
    }
}
exports.SqsProducer = SqsProducer;
//# sourceMappingURL=sqs-producer.js.map